
`timescale 1 ns / 10 ps

 module frame_gen
 (
     // User Interface
     TX_D,  
     TX_REM,   
     TX_SOF_N,     
     TX_EOF_N,

     TX_SRC_RDY_N,
     TX_DST_RDY_N,

     //UFC interface
     UFC_TX_REQ_N,
     UFC_TX_MS,
     UFC_TX_DATA,
     UFC_TX_SRC_RDY_N,
     UFC_TX_DST_RDY_N,

     //User K interface
     USER_K_DATA,
     USER_K_BLK_NO,
     USER_K_SRC_RDY_N,
     USER_K_DST_RDY_N,

     // System Interface
     CHANNEL_UP,
     USER_CLK,  
     RESET4RX,     
     RESET
 ); 
 
 //*****************************Parameter Declarations****************************
     parameter            LANE_WIDTH      = 1;
     parameter            LANE_DATA_WIDTH = (LANE_WIDTH*64);
     parameter            REM_BUS         = 5;
     parameter            REM_BITS_MAX    = (LANE_DATA_WIDTH/8);
     parameter            DATA_WIDTH      = 8;
     parameter            UFC_IFG         = 32; //UFC Inter Frame Gap
     parameter            USER_K_IFG      = 31; //USER K Inter Frame Gap
     parameter            USER_K_D        = (56*LANE_WIDTH);

     parameter            NFC_IFG         = 15; //NFC Inter Frame Gap
     parameter            NFC_PAUSE_PARAM = 15; //NFC PAUSE Value
 
 //***********************************Port Declarations*******************************
 
    //PDU Interface
     output    [0:LANE_DATA_WIDTH-1]  TX_D;
     output    [0:REM_BUS-1]          TX_REM;
     output                           TX_SOF_N;
     output                           TX_EOF_N;
     output                           TX_SRC_RDY_N;
     input                            TX_DST_RDY_N;
     
     //UFC Interface
     output                           UFC_TX_REQ_N;
     output   [0:DATA_WIDTH-1]        UFC_TX_MS;
     output   [0:LANE_DATA_WIDTH-1]   UFC_TX_DATA;
     output                           UFC_TX_SRC_RDY_N;
     input                            UFC_TX_DST_RDY_N;

     //USER K Interface
     output    [0:USER_K_D-1]         USER_K_DATA;
     output    [0:3]                  USER_K_BLK_NO;
     output                           USER_K_SRC_RDY_N;
     input                            USER_K_DST_RDY_N;

     // System Interface
       input                            CHANNEL_UP; 
       input                            USER_CLK; 
       input                            RESET4RX;
       input                            RESET;  

 //***************************External Register Declarations***************************
 
       reg                              TX_SRC_RDY_N; 
       reg                              TX_SOF_N; 
       reg                              TX_EOF_N; 
       reg     [0:REM_BUS-1]          TX_REM;
 
 //***************************Internal Register Declarations*************************** 
 
       reg                   [0:15]     pdu_lfsr_r; 
///       reg                   [0:15]     pdu_lfsr_r2; 
       reg                   [0:7]      ifg_size_r; 
       reg                   first_tx_dst_rdy_n;
       reg                   [0:3]      frame_size_r; 
       reg                   [0:3]      bytes_sent_r; 
       reg    [0:REM_BUS-1]           rem_r;
 
 //State registers for one-hot state machine
       reg                              idle_r; 
       reg                              single_cycle_frame_r; 
       reg                              sof_r; 
       reg                              data_cycle_r; 
       reg                              eof_r; 

 
 //*********************************Wire Declarations**********************************
       wire                              ifg_done_c; 
    // Next state signals for one-hot state machine
       wire                              next_idle_c; 
       wire                              next_single_cycle_frame_c; 
       wire                              next_sof_c; 
       wire                              next_data_cycle_c; 
       wire                              next_eof_c; 
 
     //State registers for UFC state machine
     reg                              ufc_start_r;
     reg                              ufc_req_r;
     reg                              ufc_data_r;
     reg                              ufc_idle_r;
     reg                              ufc_ifg_r;
     reg       [0:15]                 ufc_lfsr_r;
     reg       [0:7]                  ufc_msg_cnt;
     reg       [0:7]                  ufc_bytes_tobesent_r;  
     reg       [0:7]                  ufc_ms_count;
     wire                             resetUFC;
 
     //Next state registers for UFC state machine
     wire                             next_ufc_start_c;
     wire                             next_ufc_req_c;
     wire                             next_ufc_data_c;
     wire                             next_ufc_idle_c;
     wire                             next_ufc_ifg_c;
     
     reg       [2:0]                  count4reset4rx = 'b0; 
     wire                             dlydRESET;

    //USER K interface signals 
     reg        [3:0]                 user_k_cnt;
     reg                              USER_K_SRC_RDY_N;
     reg        [0:55]                user_k_data_mem[0:8];
     reg        [0:55]                user_k_data_r; 
     reg        [0:3]                 USER_K_BLK_NO;

     wire                             ufc_tx_src_rdy_int;
     reg            [7:0]             count_for_fc_en;
     wire                             enable_fc_i;

 
     wire                             reset_i;
     wire                             RESET_ii;
     
     reg                              reset_i_r;

     wire                             rem_i;
     wire                             reset_i_negedge;
 
 //*********************************Main Body of Code**********************************


     assign RESET_ii = RESET || RESET4RX; 
     always @(posedge USER_CLK)
         if (RESET)
         	 count4reset4rx <=  'b0;
         else if ((count4reset4rx != 'b0) && (ufc_ms_count == 'b0) && CHANNEL_UP)
         	 count4reset4rx <=  count4reset4rx - 1'b1;
         else if (RESET4RX && (ufc_ms_count != 'b0) && CHANNEL_UP)
         	 count4reset4rx <=  3'b111;
    
    assign dlydRESET = RESET || ((count4reset4rx < 3'd7) && (count4reset4rx > 3'd0)); 

    assign resetUFC = (!CHANNEL_UP)? reset_i: dlydRESET; 


    assign reset_i = RESET || (!CHANNEL_UP) || RESET4RX; 


     //Generate random data using XNOR feedback LFSR
     always @(posedge USER_CLK)
       if(reset_i)
        begin
         pdu_lfsr_r <=      16'hABCD;  //random seed value
        end

       else if(!TX_DST_RDY_N && !idle_r)
        begin
        pdu_lfsr_r  <=      {!{pdu_lfsr_r[3]^pdu_lfsr_r[12]^pdu_lfsr_r[14]^pdu_lfsr_r[15]}, 
                                 pdu_lfsr_r[0:14]};
        end
     
     //Connect TX_D to the pdu_lfsr_r register
     assign  TX_D    = {LANE_WIDTH*4{pdu_lfsr_r}};
 
 
 
     //Use a freerunning counter to determine the IFG
     always @(posedge USER_CLK)
         if(RESET_ii)
             ifg_size_r      <=      8'h0;
         else
             ifg_size_r      <=      ifg_size_r + 1;
 
     //IFG is done when ifg_size register is 0
     assign  ifg_done_c  =   (ifg_size_r[5:7] == 3'h0);
     //assign  TX_REM  = (pdu_lfsr_r[0:REM_BUS-1]<REM_BITS_MAX) ? 
     //                   pdu_lfsr_r[0:REM_BUS-1]:{REM_BUS{1'b0}};

      always @ (posedge USER_CLK)
        begin
         if(RESET_ii)
            rem_r  <=  {REM_BUS{1'b0}};	    
	 else 
	  begin  
           if(rem_r ==  REM_BITS_MAX)
            rem_r  <=  {REM_BUS{1'b0}};	    
           else
            rem_r  <=  rem_r + 1'b1;
	  end 
        end

        // ---- First Data testing -----
        always @(posedge USER_CLK)
            reset_i_r <=  reset_i;

        assign reset_i_negedge = !reset_i && reset_i_r;
        

       always @ (posedge USER_CLK)
         TX_REM <=  rem_r;

 
     //Use a counter to determine the size of the next frame to send
     always @(posedge USER_CLK)
         if(RESET_ii)
             frame_size_r    <=      4'h0;
         else if(single_cycle_frame_r || eof_r)
             frame_size_r    <=      frame_size_r + 1;
 
     //Use a second counter to determine how many bytes of the frame have already been sent
     always @(posedge USER_CLK)
         if(RESET_ii)
             bytes_sent_r    <=      4'h0;
         else if(sof_r)
             bytes_sent_r    <=      4'h1;
         else if(!TX_DST_RDY_N && !idle_r)
             bytes_sent_r    <=      bytes_sent_r + 1;
 
     //_____________________________ Framing State machine______________________________ 
     //Use a state machine to determine whether to start a frame, end a frame, send
     //data or send nothing
     
     //State registers for 1-hot state machine
     always @(posedge USER_CLK)
         if(reset_i)
         begin
             idle_r                  <=      1'b1;
             single_cycle_frame_r    <=      1'b0;
             sof_r                   <=      1'b0;
             data_cycle_r            <=      1'b0;
             eof_r                   <=      1'b0;
         end
         else if(!TX_DST_RDY_N)
         begin
             idle_r                  <=      next_idle_c;
             single_cycle_frame_r    <=      next_single_cycle_frame_c;
             sof_r                   <=      next_sof_c;
             data_cycle_r            <=      next_data_cycle_c;
             eof_r                   <=      next_eof_c;
         end
 
 
     //Nextstate logic for 1-hot state machine
     assign  next_idle_c                 =   !ifg_done_c &&
                                             (single_cycle_frame_r || eof_r || idle_r);
 
     assign  next_single_cycle_frame_c   =   (ifg_done_c && (frame_size_r == 0)) &&
                                             (idle_r || single_cycle_frame_r || eof_r);
 
     assign  next_sof_c                  =   (ifg_done_c && (frame_size_r != 0)) &&
                                             (idle_r || single_cycle_frame_r || eof_r);
 
     assign  next_data_cycle_c           =   (frame_size_r != bytes_sent_r) &&
                                             (sof_r || data_cycle_r);
 
     assign  next_eof_c                  =   (frame_size_r == bytes_sent_r) &&
                                             (sof_r || data_cycle_r);
 
     //Output logic for 1-hot state machine
     always @(posedge USER_CLK)
         if(reset_i)
         begin
             TX_SOF_N        <=      1'b1;
             TX_EOF_N        <=      1'b1;
             TX_SRC_RDY_N    <=      1'b1;
         end
         else if(!TX_DST_RDY_N)
         begin
             TX_SOF_N        <=      !(sof_r || single_cycle_frame_r);
             TX_EOF_N        <=      !(eof_r || single_cycle_frame_r);
             TX_SRC_RDY_N    <=      idle_r;
         end

     always @(posedge USER_CLK)
        if (!CHANNEL_UP)
           first_tx_dst_rdy_n <=  1'b0; 
        else if(!TX_DST_RDY_N && CHANNEL_UP)
           first_tx_dst_rdy_n <=  1'b1; 


     always @(posedge USER_CLK)
        if (!first_tx_dst_rdy_n)
           count_for_fc_en <=  8'h0;
        else if(count_for_fc_en == 8'hff)
           count_for_fc_en <=  count_for_fc_en;
        else
           count_for_fc_en <=  count_for_fc_en + 1'b1;

     assign enable_fc_i = ((count_for_fc_en == 8'hff) ? 1'b1 : 1'b0);


     /***********************UFC data generation***********************/
  
     //UFC Message counter  
     always @(posedge USER_CLK)
       if(!CHANNEL_UP)
         ufc_msg_cnt         <=      8'h00;
       else if(ufc_req_r) 
         ufc_msg_cnt         <=      ufc_msg_cnt + 1;
 
     //Counter to determine still how many UFC bytes to be transmitted
     always @(posedge USER_CLK)
       if(resetUFC) 
         ufc_bytes_tobesent_r    <=      8'd00;
       else if(ufc_req_r)
         ufc_bytes_tobesent_r    <=      UFC_TX_MS;
       else if(ufc_data_r)
         ufc_bytes_tobesent_r    <=      ufc_bytes_tobesent_r - 8'd32;
 
     //LFSR for UFC DATA 
     always @(posedge USER_CLK)
       if(resetUFC)
         ufc_lfsr_r  <=   16'hEFAC;  //random seed value
//       else if(!UFC_TX_DST_RDY_N)
       else if(ufc_req_r)
         ufc_lfsr_r  <=   {!{ufc_lfsr_r[3]^ufc_lfsr_r[12]^ufc_lfsr_r[14]^ufc_lfsr_r[15]}, 
                               ufc_lfsr_r[0:14]};
 
     assign UFC_TX_REQ_N     = !ufc_req_r;
     assign UFC_TX_MS        = ufc_msg_cnt;
     assign UFC_TX_DATA      = {LANE_WIDTH*4{ufc_lfsr_r}};

     assign ufc_tx_src_rdy_int = ufc_req_r ? 1'b0 : (ufc_ms_count > 8'd0) ? 1'b0 : 1'b1;
               
     always @(posedge USER_CLK)
           if(!CHANNEL_UP)
             ufc_ms_count <=  8'h0;
           else if(ufc_req_r)
             ufc_ms_count <=  ufc_msg_cnt + 1'b1;
           else if(!ufc_tx_src_rdy_int & !UFC_TX_DST_RDY_N & (ufc_ms_count >= 8'd32))
             ufc_ms_count <=  ufc_ms_count -8'd32;
           else if(!ufc_tx_src_rdy_int & !UFC_TX_DST_RDY_N & (ufc_ms_count >= 8'd0))
             ufc_ms_count <=  8'h0;

     assign UFC_TX_SRC_RDY_N = (!ufc_tx_src_rdy_int & !UFC_TX_DST_RDY_N & (ufc_ms_count == 8'd0)) ? 1'b1 : ufc_tx_src_rdy_int;
 
     //UFC State machine
     always @(posedge USER_CLK)
         if((!CHANNEL_UP) && (!first_tx_dst_rdy_n))
         begin
             ufc_start_r             <=      1'b1;
             ufc_req_r               <=      1'b0;
             ufc_data_r              <=      1'b0;
             ufc_idle_r              <=      1'b0;
             ufc_ifg_r               <=      1'b0;
         end
         else if(enable_fc_i)
         begin
             ufc_start_r             <=      next_ufc_start_c;
             ufc_req_r               <=      next_ufc_req_c;
             ufc_data_r              <=      next_ufc_data_c;
             ufc_idle_r              <=      next_ufc_idle_c;
             ufc_ifg_r               <=      next_ufc_ifg_c;
         end
    
     //UFC FSM will be active once Aurora reaches CHANNEL UP state
     assign next_ufc_start_c   = ufc_ifg_r && (pdu_lfsr_r[0:7]==UFC_IFG);   	
 
     //UFC FSM generate the UFC REQ to its channel partner
     assign next_ufc_req_c     =  ufc_start_r;
 
     //UFC FSM stays in data state till it receives UFC_DST_RDY_N from
     //its channel partner
     assign next_ufc_data_c    =  (ufc_idle_r || ufc_data_r ) && (!UFC_TX_DST_RDY_N);
 
     //UFC FSM moves into idle stage when it hasn't received UFC_DST_RDY signal 
     //while it is transmitting UFC data and have some more UFC data to be transmiited 
     assign next_ufc_idle_c    =  (ufc_req_r || 
                                  (ufc_idle_r || (ufc_data_r && (ufc_bytes_tobesent_r>=8'd32)))) && 
                                   UFC_TX_DST_RDY_N;
 
     //UFC FSM comes to this state after all UFC data has been transmitted
     assign next_ufc_ifg_c     =  ufc_ifg_r && (!(pdu_lfsr_r[0:7]==UFC_IFG))|| 
                                  (ufc_data_r && (ufc_bytes_tobesent_r<=8'd31));

     /**************USER K data generation**********/
 
     //Memory with randomly chosen nine USER K values
     always @ (posedge USER_CLK)
       begin
         user_k_data_mem[0]       <=   {16'hAB40, 40'h0};
         user_k_data_mem[1]       <=   {16'hCD90, 40'h0};
         user_k_data_mem[2]       <=   {16'h1230, 40'h0};
         user_k_data_mem[3]       <=   {16'h5678, 40'h0};
         user_k_data_mem[4]       <=   {16'h1EBC, 40'h0};
         user_k_data_mem[5]       <=   {16'hDEF0, 40'h0};
         user_k_data_mem[6]       <=   {16'h2D7F, 40'h0};
         user_k_data_mem[7]       <=   {16'h78FD, 40'h0};
         user_k_data_mem[8]       <=   {16'hAAE1, 40'h0};
       end 
 
     //Counter to generate the USER K BLK NO
     always @ (posedge USER_CLK)
       if(reset_i)
         user_k_cnt    <=   4'h0;
       else if(user_k_cnt[3] && (!USER_K_DST_RDY_N) && (!USER_K_SRC_RDY_N))	   
         user_k_cnt    <=   4'h0;
       else if(!USER_K_DST_RDY_N && !USER_K_SRC_RDY_N)
         user_k_cnt    <=   user_k_cnt + 1;
 
     //Send USER K Data with respect USER K BLK NO
     always @ (posedge USER_CLK)
       if(reset_i)
         user_k_data_r    <=   user_k_data_mem[0];
       else
         user_k_data_r    <=   user_k_data_mem[user_k_cnt];
 
     assign USER_K_DATA   = {LANE_WIDTH{user_k_data_r}};
 
     always @ (posedge USER_CLK)
       USER_K_BLK_NO <=  user_k_cnt;
 
     //Logic for USER_K_SRC_RDY_N signal generation 
     always @ (posedge USER_CLK)
       if(reset_i || pdu_lfsr_r[0:7]!=USER_K_IFG)
         USER_K_SRC_RDY_N    <=   1'b1;
       else if(pdu_lfsr_r[0:7]==USER_K_IFG)
         USER_K_SRC_RDY_N    <=   1'b0;


 
 endmodule 
