`timescale 1 ns / 1 ps
module tb_top();

reg             rst  = 1;
reg             pcs_tx_clk = 0;
reg             pcs_rx_clk = 0;
reg             sys_clk = 0;
reg             init_clk = 0;

localparam      SYS_CLK_PERIOD         = (20.48/2);
localparam      INIT_CLK_PERIOD        = (20/2); 

parameter       LANE_WIDTH             = 1;
parameter       DATA_WIDTH             = 64*LANE_WIDTH;
parameter       KEEP_WIDTH             = 8*LANE_WIDTH;
parameter       USERK_WIDTH            = 56*LANE_WIDTH;
parameter       USERK_BLK_WIDTH        = 4*LANE_WIDTH;

parameter       REM_WIDTH              = (LANE_WIDTH == 1) ? 3 : (LANE_WIDTH == 2) ? 4 : 
                                         ((LANE_WIDTH == 3) | (LANE_WIDTH == 4)) ? 5 : 6;

always #SYS_CLK_PERIOD   pcs_tx_clk = ~pcs_tx_clk;        
always #SYS_CLK_PERIOD   pcs_rx_clk = ~pcs_rx_clk;        
always #SYS_CLK_PERIOD   sys_clk    = ~sys_clk;        
always #INIT_CLK_PERIOD  init_clk   = ~init_clk;

initial begin
    rst = 1;
    #1000;
    rst = 0;
    #100000;
end

wire [LANE_WIDTH-1:0]                 serdes_lnx_tx_vld;
wire [80*LANE_WIDTH-1:0]              serdes_lnx_txdata;

wire [LANE_WIDTH-1:0]                 serdes_lnx_rx_vld;
wire [88*LANE_WIDTH-1:0]              serdes_lnx_rxdata;

// --------- user interface TX Data interface  
wire [64*LANE_WIDTH-1:0]              user_tx_data;
wire                                  user_tx_valid;
wire                                  user_tx_ready;
wire [8*LANE_WIDTH-1:0]               user_tx_strb;
wire                                  user_tx_last;

// --------- user interface RX Data interface  
wire [64*LANE_WIDTH-1:0]              user_rx_data;
wire                                  user_rx_valid;
wire [8*LANE_WIDTH-1:0]               user_rx_strb;
wire                                  user_rx_last;

// --------- User Flow Control interface  
wire                                  ufc_tx_req;
wire [7:0]                            ufc_tx_ms;
wire [64*LANE_WIDTH-1:0]              ufc_tx_data;
wire                                  ufc_tx_valid;
wire                                  ufc_tx_ready;

wire [64*LANE_WIDTH-1:0]              ufc_rx_data;
wire [8*LANE_WIDTH-1:0]               ufc_rx_strb;
wire                                  ufc_rx_last;
wire                                  ufc_rx_valid;
wire                                  ufc_in_progress;

// --------- user interface TX K-Core interface 
wire [64*LANE_WIDTH-1:0]              user_tx_k_data;
wire                                  user_tx_k_valid;
wire                                  user_tx_k_ready;

// --------- user interface RX K-Core interface 
wire [64*LANE_WIDTH-1:0]              user_rx_k_data;
wire                                  user_rx_k_valid;

// --------- Status interface  
wire                                  hard_err;
wire                                  soft_err;
wire                                  channel_up;
wire [LANE_WIDTH-1:0]                 lane_up;
wire                                  tx_channel_up;
wire                                  rx_channel_up;

wire                                  crc_pass_fail_n;
wire                                  crc_valid;
    
//-------- User define serdes control
wire                                  gt_pll_ok;  

wire                                  tx_sys_reset;
wire                                  reset_tx;

wire                                  reset_rx;
wire                                  rx_sys_reset;

wire [0:64*LANE_WIDTH-1]              tx_d;       
wire                                  tx_sof_n;    
wire                                  tx_eof_n;    
wire [0:REM_WIDTH-1]                  tx_rem;     
wire                                  tx_src_rdy_n;
wire                                  tx_dst_rdy_n;

wire [0:64*LANE_WIDTH-1]              ufc_ll_tx_data;     
wire [0:7]                            ufc_ll_tx_ms;       
wire                                  ufc_ll_tx_src_rdy_n;
wire                                  ufc_ll_tx_dst_rdy_n;
wire                                  ufc_tx_req_n;


wire [0:64*LANE_WIDTH-1]              user_k_ll_data;  
wire                                  user_k_src_rdy_n;
wire                                  user_k_dst_rdy_n;

wire [0:4-1]                          user_k_blk_no;
wire [0:56*LANE_WIDTH-1]              user_k_data;


wire [0:DATA_WIDTH-1]                 rx_d;        
wire                                  rx_sof_n;    
wire                                  rx_eof_n;    
wire [0:REM_WIDTH-1]                  rx_rem;   
wire                                  rx_src_rdy_n;

wire [0:DATA_WIDTH-1]                 ufc_ll_rx_data;       
wire                                  ufc_ll_rx_sof_n;     
wire                                  ufc_ll_rx_eof_n;     
wire [0:REM_WIDTH-1]                  ufc_ll_rx_rem;       
wire                                  ufc_ll_rx_src_rdy_n; 

wire [0:56*LANE_WIDTH-1]              ll_rx_user_k_data;     
wire                                  ll_rx_user_k_src_rdy_n;
wire [0:4*LANE_WIDTH-1]               ll_rx_user_k_blk_no;
wire [0:64*LANE_WIDTH-1]              rx_user_k_data;
wire  [0:7]                           user_k_err_count;

wire  [7:0]                           ufc_err_count;
reg   [7:0]                           crc_err_count;
wire  [7:0]                           data_err_count;

generate begin
    assign serdes_lnx_rx_vld = serdes_lnx_tx_vld;
    genvar i;
    for (i = 0; i < LANE_WIDTH; i = i + 1) begin : gen_serdes_rx
        assign serdes_lnx_rxdata[i*88+87:i*88]     =   {8'h0,serdes_lnx_txdata[i*80+79:i*80]};
    end
end
endgenerate 

    assign tx_channel_up = channel_up;
    assign rx_channel_up = channel_up;
    assign reset_tx      = tx_sys_reset | (!channel_up);
    assign reset_rx      = rx_sys_reset | (!channel_up);

RoraLink_64B66B_Top u_RoraLink_64B66B_Top
(
    // --------- Clock & Reset
    .sys_rst_i                             ( rst                              ),
    .sys_clk_i                             ( sys_clk                          ),
    .init_clk_i                            ( init_clk                         ),

    // --------- user interface TX Data interface  
    .user_tx_data_i                        ( user_tx_data                     ), 
    .user_tx_valid_i                       ( user_tx_valid                    ),
    .user_tx_ready_o                       ( user_tx_ready                    ), 
    .user_tx_strb_i                        ( user_tx_strb                     ),
    .user_tx_last_i                        ( user_tx_last                     ),

    // --------- user interface RX Data interface  
    .user_rx_data_o                        ( user_rx_data                     ),
    .user_rx_valid_o                       ( user_rx_valid                    ),
    .user_rx_strb_o                        ( user_rx_strb                     ),
    .user_rx_last_o                        ( user_rx_last                     ),

    // --------- User Flow Control interface  
    .ufc_tx_req_i                          ( ufc_tx_req                       ),
    .ufc_tx_ms_i                           ( ufc_tx_ms                        ),
    .ufc_tx_data_i                         ( ufc_tx_data                      ),
    .ufc_tx_valid_i                        ( ufc_tx_valid                     ),
    .ufc_tx_ready_o                        ( ufc_tx_ready                     ),

    .ufc_rx_data_o                         ( ufc_rx_data                      ),
    .ufc_rx_strb_o                         ( ufc_rx_strb                      ),
    .ufc_rx_last_o                         ( ufc_rx_last                      ),
    .ufc_rx_valid_o                        ( ufc_rx_valid                     ),
    .ufc_in_progress_o                     ( ufc_in_progress                  ),

    // --------- user interface TX K-Core interface 
    .user_tx_k_data_i                      ( user_tx_k_data                   ),
    .user_tx_k_valid_i                     ( user_tx_k_valid                  ),
    .user_tx_k_ready_o                     ( user_tx_k_ready                  ),

    // --------- user interface RX K-Core interface 
    .user_rx_k_data_o                      ( user_rx_k_data                   ),
    .user_rx_k_valid_o                     ( user_rx_k_valid                  ),

    // --------- Status interface  
    .hard_err_o                            ( hard_err                         ),
    .soft_err_o                            ( soft_err                         ),
    .channel_up_o                          ( channel_up                       ),
    .lane_up_o                             ( lane_up                          ),

    .crc_pass_fail_n_o                     ( crc_pass_fail_n                  ),
    .crc_valid_o                           ( crc_valid                        ),
    
    //-------- User define serdes control
    .gt_pma_rstn_i                         ( 1'b1                             ),///gt_pma_rstn                       ),
    .gt_pll_ok_o                           ( gt_pll_ok                        ),  
    .gt_pcs_tx_clk_o                       (                                  ),

    .gt_pcs_tx_rst_i                       ( 1'b0                             ),///gt_pcs_tx_rst                 ),
    .gt_tx_fifo_wrusewd_o                  (                                  ),                                 

    .gt_rx_align_link_o                    (                                  ),
    .gt_rx_pma_lock_o                      (                                  ),
    .gt_pcs_rx_clk_o                       (                                  ),                         
    .gt_pcs_rx_rst_i                       ( 1'b0                             ),///gt_pcs_rx_rst                 ),
    .gt_rx_fifo_rdusewd_o                  (                                  ),

    .tx_sys_reset_o                        ( tx_sys_reset                     ), 

    .rx_sys_reset_o                        ( rx_sys_reset                     ),
   
    //-------- Serdes interface    
    .serdes_pcs_tx_rst_o                   (                                  ),///LANE#_PCS_TX_RST
    .serdes_lanex_fabric_tx_clk_o          (                                  ),///LANE#_FABRIC_TX_CLK
    .serdes_lanex_tx_if_fifo_afull_i       ( {LANE_WIDTH{1'b0}}               ),///LANE#_TX_IF_FIFO_AFULL
    .serdes_lnx_tx_vld_o                   ( serdes_lnx_tx_vld                ),///FABRIC_LN#_TX_VLD_IN
    .serdes_lnx_txdata_o                   ( serdes_lnx_txdata                ),///FABRIC_LN#_TXDATA_I   
    .serdes_tx_if_fifo_wrusewd_i           ( 'd0                              ),///LANE#_TX_IF_FIFO_WRUSEWD 
 
    .serdes_pcs_rx_rst_o                   (                                  ),///LANE#_PCS_RX_RST
    .serdes_lanex_fabric_rx_clk_o          (                                  ),///LANE#_FABRIC_RX_CLK  
    .serdes_lanex_rx_if_fifo_aempty_i      ( {LANE_WIDTH{1'b0}}               ),///LANE#_RX_IF_FIFO_AEMPTY
    .serdes_lanex_rx_if_fifo_rden_o        (                                  ),///LANE#_RX_IF_FIFO_RDEN
    .serdes_lnx_rx_vld_i                   ( serdes_lnx_rx_vld                ),///FABRIC_LN#_RX_VLD_OUT
    .serdes_lnx_rxdata_i                   ( serdes_lnx_rxdata                ),///FABRIC_LN#_RXDATA_O
    .serdes_rx_if_fifo_rdusewd_i           ( 'd0                              ),///LANE#_RX_IF_FIFO_RDUSEWD    
    .serdes_lnx_pma_rx_lock_i              ( {LANE_WIDTH{1'b1}}               ),///FABRIC_LN#_PMA_RX_LOCK_O
    .serdes_lanex_align_link_i             ( {LANE_WIDTH{1'b1}}               ),///LANE#_ALIGN_LINK
    .serdes_lanex_k_lock_i                 ( {LANE_WIDTH{1'b1}}               ),///LANE#_K_LOCK
    .serdes_lnx_rstn_o                     (                                  ),///FABRIC_LN#_RSTN_I
    .serdes_lanex_pcs_tx_fabric_clk_i      ( {LANE_WIDTH{pcs_tx_clk}}         ),///LANE#_PCS_TX_O_FABRIC_CLK
    .serdes_lanex_pcs_rx_fabric_clk_i      ( {LANE_WIDTH{pcs_rx_clk}}         ),///LANE#_PCS_RX_O_FABRIC_CLK
    .serdes_q0_cmu1_ok_i                   ( 1'b1                             ),///QUAD0 FABRIC_CMU1_OK_O
    .serdes_q0_cmu0_ok_i                   ( 1'b1                             ),///QUAD0 FABRIC_CMU_OK_O
    .serdes_q1_cmu1_ok_i                   ( 1'b1                             ),///QUAD1 FABRIC_CMU1_OK_O
    .serdes_q1_cmu0_ok_i                   ( 1'b1                             ),///QUAD1 FABRIC_CMU_OK_O
    .serdes_lanex_cmu_ok_i                 ( {LANE_WIDTH{1'b1}}               ) ///FABRIC_LANE#_CMU_OK_O
);

ll_to_axi_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( KEEP_WIDTH                       ),
    .USE_4_NFC                             ( 0                                ),
    .REM_WIDTH                             ( REM_WIDTH                        )
)                                         
u_data_ll_to_axi                          
(                                         
    // LocalLink input Interface          
    .LL_IP_DATA                            ( tx_d                             ),                                                         
    .LL_IP_SOF_N                           ( tx_sof_n                         ),
    .LL_IP_EOF_N                           ( tx_eof_n                         ),
    .LL_IP_REM                             ( tx_rem                           ),
    .LL_IP_SRC_RDY_N                       ( tx_src_rdy_n                     ),
    .LL_OP_DST_RDY_N                       ( tx_dst_rdy_n                     ),

    // AXI4-S output signals              
    .AXI4_S_OP_TVALID                      ( user_tx_valid                    ),
    .AXI4_S_OP_TDATA                       ( user_tx_data                     ),
    .AXI4_S_OP_TKEEP                       ( user_tx_strb                     ),
    .AXI4_S_OP_TLAST                       ( user_tx_last                     ),
    .AXI4_S_IP_TREADY                      ( user_tx_ready                    ),
    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_tx                         )
);

    assign ufc_tx_req = !ufc_tx_req_n;
ll_to_axi_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( 8                                ),
    .USE_4_NFC                             ( 2                                ),
    .REM_WIDTH                             ( 8                                )
)
u_ufc_ll_to_axi
(
    .LL_IP_DATA                            ( ufc_ll_tx_data                   ),
    .LL_IP_SOF_N                           (                                  ),
    .LL_IP_EOF_N                           (                                  ),
    .LL_IP_REM                             ( ufc_ll_tx_ms                     ),
    .LL_IP_SRC_RDY_N                       ( ufc_ll_tx_src_rdy_n              ),
    .LL_OP_DST_RDY_N                       ( ufc_ll_tx_dst_rdy_n              ),

    // AXI4-S output signals                                                  
    .AXI4_S_OP_TVALID                      ( ufc_tx_valid                     ),
    .AXI4_S_OP_TDATA                       ( ufc_tx_data                      ),
    .AXI4_S_OP_TKEEP                       ( ufc_tx_ms                        ),
    .AXI4_S_OP_TLAST                       (                                  ),

    .AXI4_S_IP_TREADY                      ( ufc_tx_ready                     ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_tx                         )
);

generate begin
    genvar i;
    for (i = 0; i < LANE_WIDTH; i = i + 1) begin : gen_user_tx_k
        assign user_k_ll_data[LANE_WIDTH*64-64-i*64:LANE_WIDTH*64-i*64-1'b1]     =   {4'h0,user_k_blk_no[0:3],user_k_data[56*i:56*i+55]};
    end
end
endgenerate 

ll_to_axi_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( KEEP_WIDTH                       ),
    .USE_4_NFC                             ( 3                                ),
    .REM_WIDTH                             ( REM_WIDTH                        )
)
u_userk_ll_to_axi
(
    .LL_IP_DATA                            ( user_k_ll_data                   ),
    .LL_IP_SOF_N                           (                                  ),
    .LL_IP_EOF_N                           (                                  ),
    .LL_IP_REM                             (                                  ),
    .LL_IP_SRC_RDY_N                       ( user_k_src_rdy_n                 ),
    .LL_OP_DST_RDY_N                       ( user_k_dst_rdy_n                 ),

    // AXI4-S output signals                                                     
    .AXI4_S_OP_TVALID                      ( user_tx_k_valid                  ),
    .AXI4_S_OP_TDATA                       ( user_tx_k_data                   ),
    .AXI4_S_OP_TKEEP                       (                                  ),
    .AXI4_S_OP_TLAST                       (                                  ),

    .AXI4_S_IP_TREADY                      ( user_tx_k_ready                  ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_tx                         )
);

// Frame Generator to provide with input to TX_Stream
frame_gen #
(
    .REM_BUS                               ( REM_WIDTH                        )
)
u_frame_gen
(
    .TX_D                                  ( tx_d                             ),                                                            
    .TX_SOF_N                              ( tx_sof_n                         ),
    .TX_EOF_N                              ( tx_eof_n                         ),
    .TX_REM                                ( tx_rem                           ),                                                                       
    .TX_SRC_RDY_N                          ( tx_src_rdy_n                     ),
    .TX_DST_RDY_N                          ( tx_dst_rdy_n                     ),
                                                             
    //UFC TX interface                                                        
    .UFC_TX_REQ_N                          ( ufc_tx_req_n                     ),
    .UFC_TX_MS                             ( ufc_ll_tx_ms                     ),
    .UFC_TX_DATA                           ( ufc_ll_tx_data                   ),
    .UFC_TX_SRC_RDY_N                      ( ufc_ll_tx_src_rdy_n              ),
    .UFC_TX_DST_RDY_N                      ( ufc_ll_tx_dst_rdy_n              ),                                                                   
                                                           
    //User K TX interface                                                    
    .USER_K_DATA                           ( user_k_data                      ),
    .USER_K_BLK_NO                         ( user_k_blk_no                    ),
    .USER_K_SRC_RDY_N                      ( user_k_src_rdy_n                 ),
    .USER_K_DST_RDY_N                      ( user_k_dst_rdy_n                 ),

    .CHANNEL_UP                            ( tx_channel_up                    ),
    .RESET4RX                              ( 1'b0                             ),
    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_tx                         )
);

axi_to_ll_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( KEEP_WIDTH                       ),
    .ISUFC                                 ( 0                                ),
    .REM_WIDTH                             ( REM_WIDTH                        )
)
u_data_axi_to_ll
(
    // AXI4-S input signals                                                     
    .AXI4_S_IP_TX_TVALID                   ( user_rx_valid                    ),
    .AXI4_S_IP_TX_TREADY                   (                                  ),
    .AXI4_S_IP_TX_TDATA                    ( user_rx_data                     ),
    .AXI4_S_IP_TX_TKEEP                    ( user_rx_strb                     ),
    .AXI4_S_IP_TX_TLAST                    ( user_rx_last                     ),

    // LocalLink output Interface
    .LL_OP_DATA                            ( rx_d                             ),
    .LL_OP_SOF_N                           ( rx_sof_n                         ),
    .LL_OP_EOF_N                           ( rx_eof_n                         ),
    .LL_OP_REM                             ( rx_rem                           ),
    .LL_OP_SRC_RDY_N                       ( rx_src_rdy_n                     ),
    .LL_IP_DST_RDY_N                       ( 1'b0                             ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_rx                         ),
    .CHANNEL_UP                            ( rx_channel_up                    )
);

axi_to_ll_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( KEEP_WIDTH                       ),
    .ISUFC                                 ( 1                                ),
    .REM_WIDTH                             ( REM_WIDTH                        )
)
u_ufc_axi_to_ll
(
    // AXI4-S input signals                                                     
    .AXI4_S_IP_TX_TVALID                   ( ufc_rx_valid                     ),
    .AXI4_S_IP_TX_TREADY                   (                                  ),
    .AXI4_S_IP_TX_TDATA                    ( ufc_rx_data                      ),                                                         
    .AXI4_S_IP_TX_TKEEP                    ( ufc_rx_strb                      ),
    .AXI4_S_IP_TX_TLAST                    ( ufc_rx_last                      ),                                                                  
    // LocalLink output Interface                                            
    .LL_OP_DATA                            ( ufc_ll_rx_data                   ),                                                        
    .LL_OP_SOF_N                           ( ufc_ll_rx_sof_n                  ),
    .LL_OP_EOF_N                           ( ufc_ll_rx_eof_n                  ),
    .LL_OP_REM                             ( ufc_ll_rx_rem                    ),                                                                      
    .LL_OP_SRC_RDY_N                       ( ufc_ll_rx_src_rdy_n              ),
    .LL_IP_DST_RDY_N                       ( 1'b0                             ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_rx                         ),
    .CHANNEL_UP                            ( rx_channel_up                    )
);

axi_to_ll_reg #
(
    .DATA_WIDTH                            ( DATA_WIDTH                       ),
    .STRB_WIDTH                            ( KEEP_WIDTH                       ),
    .ISUFC                                 ( 0                                ),
    .REM_WIDTH                             ( REM_WIDTH                        )
)
u_userk_axi_to_ll
(
    // AXI4-S input signals                                                     
    .AXI4_S_IP_TX_TVALID                   ( user_rx_k_valid                  ),
    .AXI4_S_IP_TX_TREADY                   (                                  ),
    .AXI4_S_IP_TX_TDATA                    ( user_rx_k_data                   ),                                                                     
    .AXI4_S_IP_TX_TKEEP                    ( {KEEP_WIDTH{1'b0}}               ),
    .AXI4_S_IP_TX_TLAST                    ( 1'b0                             ),

    // LocalLink output Interface                                            
    .LL_OP_DATA                            ( rx_user_k_data                   ),     
    .LL_OP_SOF_N                           (                                  ),
    .LL_OP_EOF_N                           (                                  ),
    
    .LL_OP_REM                             (                                  ),                                                                     
    .LL_OP_SRC_RDY_N                       ( ll_rx_user_k_src_rdy_n           ),
    .LL_IP_DST_RDY_N                       ( 1'b0                             ),

    .USER_CLK                              ( sys_clk                          ),
    .RESET                                 ( reset_rx                         ),
    .CHANNEL_UP                            ( rx_channel_up                    )
);

generate begin
    genvar i;
    for (i = 0; i < LANE_WIDTH; i = i + 1) begin : gen_user_rx_k
        assign ll_rx_user_k_blk_no[LANE_WIDTH*4-4-i*4:LANE_WIDTH*4-i*4-1'b1]   =   rx_user_k_data[64*i+4:64*i+7];
        assign ll_rx_user_k_data[LANE_WIDTH*56-56-i*56:LANE_WIDTH*56-i*56-1'b1]     =   rx_user_k_data[64*i+8:64*i+63];
    end
end
endgenerate 

// Frame Checker to check output from RX_Stream
frame_check #
(
    .REM_BUS                               ( REM_WIDTH                        )
)
u_frame_check
(
    .RX_D                                  ( rx_d                             ),
    .RX_REM                                ( rx_rem                           ),
    .RX_SOF_N                              ( rx_sof_n                         ),
    .RX_EOF_N                              ( rx_eof_n                         ),
    .RX_SRC_RDY_N                          ( rx_src_rdy_n                     ),
    .DATA_ERR_COUNT                        ( data_err_count                   ),

    //User Flow Control Interface
    .UFC_RX_DATA                           ( ufc_ll_rx_data                   ),
    .UFC_RX_REM                            ( ufc_ll_rx_rem                    ),                                                      
    .UFC_RX_SOF_N                          ( ufc_ll_rx_sof_n                  ),
    .UFC_RX_EOF_N                          ( ufc_ll_rx_eof_n                  ),
    .UFC_RX_SRC_RDY_N                      ( ufc_ll_rx_src_rdy_n              ),
    .UFC_ERR_COUNT                         ( ufc_err_count                    ),
    .UFC_IN_PROGRESS                       ( ufc_in_progress                  ),

    // RX User K Interface
    .RX_USER_K_DATA                        ( ll_rx_user_k_data                ),
    .RX_USER_K_SRC_RDY_N                   ( ll_rx_user_k_src_rdy_n           ),
    .RX_USER_K_BLK_NO                      ( ll_rx_user_k_blk_no              ),
    .USER_K_ERR_COUNT                      ( user_k_err_count                 ),
    
    .CHANNEL_UP                            ( rx_channel_up                    ),
    .USER_CLK                              ( sys_clk                          ),
    
    .RESET4RX                              ( 1'b0                             ),

    .RESET                                 ( reset_rx                         )
);

    always @(posedge sys_clk) begin
        if(reset_rx) begin
            crc_err_count <= 8'd0;
        end
        else if(crc_valid & !crc_pass_fail_n)begin
            crc_err_count <= crc_err_count + 1'b1;
        end
    end


initial begin
    $fsdbDumpfile("tb.fsdb");
    $fsdbDumpvars;
end

initial begin 
	#7000_0000;
	$finish;
end


endmodule
